<?php
#================================================================
#  Family Tree Listing - written by J.A.Pipes, Nov 2024
#  This software is provided under the GNU General Public License
#  to be used, modified, and distributed, provided the text of
#  this header is retained and unmodified.
#  
#  A download package is available at:
#  https://pipes.family/archive/software/
#
#  This script recursively traverses a folder tree and renders
#  the data found there into a graphical representation of a
#  family tree. 
#================================================================

#================================================================
#  Variables
#================================================================
// server path 
$serverPath = "/home/userpipfam/pipes.family";
// store number of tree levels
$maxDepth = 0;

#================================================================
#  Functions
#================================================================

function printHeader() {
    print "<!-- Family Tree Listing - provided by the Pipes Family Foundation (2024) -->\n";
    print "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n";
    print "<html xmlns=\"http://www.w3.org/1999/xhtml\">\n";

    print "<head>\n";
    print "<title>Jones Family Tree Listing</title>\n";
    print "<!-- Import default stylesheet -->\n";
    print "<link rel=\"stylesheet\" href=\"tree.css\" type=\"text/css\">\n";
    print "</head>\n";

    print "<body>\n";

    print "<div class=\"tree\">\n";
}

function extractURL($element) {
    // if URL file exists
    if (file_exists($element)) {
        $lines = file($element);
        $URL = $lines[1];
        // remove "URL=" from beginning of line
        $href = substr($URL, 4);
    }
    return $href;
}

function getElements($option, $path) {
    // hold return value
    $elementList = array();

    // if files have been requested
    if($option == "files") {
        // retrieve directory of path
        $dir = new DirectoryIterator($path);
        // step through elements of directory 
        while ($dir->valid()) {
            // skip portrait photo
            if ($dir->getFilename() == "photo.jpg") {
                $dir->next();
                continue;
            }
            // skip military service file
            if ($dir->getFilename() == "veteran.txt") {
                $dir->next();
                continue;
            }
            // skip vitals file
            if ($dir->getFilename() == "vitals.txt") {
                $dir->next();
                continue;
            }
            // if element is a file
            if ($dir->isFile()) {
                // push info onto the file array
                array_push($elementList,$dir->getFilename());
            }
            // jump to next element
            $dir->next();
        }
        // sort folder list
        sort($elementList);
    }
    // if folders have been requested
    if($option == "folders") {
        // retrieve directory of path
        $dir = new DirectoryIterator($path);
        // step through elements of directory 
        while ($dir->valid()) {
            // skip . and .. folders
            if ($dir->isDot()) {
                $dir->next();
                continue;
            }
            // skip 's' folder
            if ($dir->getFilename() == "s") {
                $dir->next();
                continue;
            }
            // if element is a folder
            if ($dir->isDir()) {
                // push info onto the folder array
                array_push($elementList,$dir->getFilename());
            }
            // jump to next element
            $dir->next();
        }
        // sort folder list
        sort($elementList);
    }
    // return elements list
    return $elementList;
}

function displayPersonCard($path, $person) {
    // path to site
    global $serverPath;
    // split person into gender and name
    list($gender, $name) = explode("-", $person, 2);

    // check for photo
    if (file_exists($serverPath."/".$path."/".$person."/photo.jpg")) {
        $portrait = "<div class=\"portrait\"><img src=\"/".$path."/".$person."/photo.jpg\"></div>\n";
    }
    else {
        $portrait = "<div class=\"portrait\"><img src=\"".$gender."-photo.jpg\"></div>\n";
    }

    // check for military service
    if (file_exists($serverPath."/".$path."/".$person."/veteran.txt")) {
        // read vitals file
        $service = file_get_contents($serverPath."/".$path."/".$person."/veteran.txt");
        $badge = "<img class=\"badge\" src=\"veteran-icon.svg\" title=\"".$service."\">";
    }

	// check for vitals
    if (file_exists($serverPath."/".$path."/".$person."/vitals.txt")) {
        // read vitals file
        $stats = file($serverPath."/".$path."/".$person."/vitals.txt");
    }
    $vitals = "B:".$stats[0]."<br>\nM:".$stats[1]."<br>\nD:".$stats[2]."<br>\n";
    // get references
    $refs = getElements("files", $serverPath."/".$path."/".$person);    

    // display card
    print "<!------------------- PERSON -------------------".$path."/".$person."-->\n";
    print "<div class=\"person ".$gender."\"><span>".$name." ".$badge."</span>\n";
    print "<div class=\"bio\">\n";
    print "<div class=\"top\">".$portrait."\n";
    print "<div class=\"vitals\">\n".$vitals."</div>\n";
    print "</div><!-- end top -->\n";
    print "<div class=\"references\">References:\n<ol>\n";
    foreach ($refs as $ref) {
        // extract URL
        $URL = extractURL($serverPath."/".$path."/".$person."/".$ref);
        print "<li><a href=\"".$URL."\">".substr($ref, 0, -4)."</a></li>\n";
    }
    print "</ol>\n</div><!-- end references -->\n";
    print "</div><!-- end bio -->\n";
    print "</div><!-- end person ---------------------------->\n";
}

function displayPerson($path, $person, $depth) {
    // path to site
    global $serverPath;
    // maximum depth
    global $maxDepth;
    // array to hold all folders
    $folders = array();
    // array to hold siblings
    $siblings = array();
    // array to hold parents
    $parents = array();
    // increment depth
    $depth++;
    // get folder count
    $folderCount = sizeof(getElements("folders", $serverPath."/".$path));
    // if there are no subfolders
    if ($folderCount == 0) {
        $maxDepth = $depth;
        return;
    }

    // open listing
    print "<div class=\"generation\"><!-- ".$path." -- folder count: ".$folderCount." -->\n";
    print "<div class=\"children\">\n";

    // display children
    displayPersonCard($path, $person);

    // if siblings folder exists
    if (file_exists($serverPath."/".$path."/".$person."/s")) {
        print "<div class=\"siblings\"><!-- ".$path."/".$person."/s -->\n";
        // get siblings
        $siblings = getElements("folders", $serverPath."/".$path."/".$person."/s");
        foreach($siblings as $sibling) {
            displayPersonCard($path."/".$person."/s", $sibling);
        }
        print "</div><!-- end siblings -->\n";
    }
    print "</div><!-- end children -->\n";

    print "<div class=\"parents\"><!-- ".$path."/".$person." -->\n";
    // get parents
    $parents = getElements("folders", $serverPath."/".$path."/".$person);
    // display father
    displayPerson($path."/".$person, $parents[1], $depth);
    // display mother
    displayPerson($path."/".$person, $parents[0], $depth);
    print "</div><!-- end parents -->\n";
    // close listing
    print "</div><!-- end generation -->\n";
}

function printFooter($depth) {
    print "</div><!-- end tree -->\n";
    print "<!-- number of tree levels: [".($depth-1)."] -->\n";
    print "</body>\n";
    print "</html>\n";
}


#================================================================
#  Main
#================================================================
// output page header
printHeader();
// display listing of target path
displayPerson("archive/software/tree/demo", "M-Benjamin Jones", 0);
// output page footer
printFooter($maxDepth);

?>